/****************************************************/
/*                                                  */
/*       Likelihood*Prior for the Panel Wage Eq     */
/*                                                  */
/****************************************************/

/* filename:    lbdpm_p.g
** description: The program computes the likelihood * prior for the
**              panel wage equation
** created:     05/21/01
*/
new;
closeall;
cls;
outwidth 128;

library pgraph, lbdlib;
logfile1 = "c:\\projects\\active\\persist\\gauss\\prog_p03\\lbdpm_p1.log";
logfile2 = "c:\\projects\\active\\persist\\gauss\\prog_p03\\lbdpm_p2.log";

/********************************************************
**      List of Global Variables
**
**      ww, xx, xw, wiiw, xiix, xiiw, iw, ix, w_names, x_names, ti
**      nx, nw, nchar, nydummies
**      gamss_m0, gamss_v0, beta_m0, beta_v0
**      age_base, educ_base, gender_base, hours_base, xbaseline
*/
#include c:\projects\active\persist\gauss\prog_p03\confspec.g;

/********************************************************
**      Load "sufficient statistics" for panel data set
**
*/
lpath = "c:\\projects\\active\\persist\\data_p\\spec" $+ ftos(mspec,"%*.*lf",1,0);
loadm path = ^lpath wiiw, xiiw, xiix, iw, ix,
                    ww, xx, xw, w_names, x_names, ti,
                    nchar, nydummies;
nx = cols(xx);
nw = cols(ww);
"Data Matrices successfully loaded";
"";
"Number of Observations: "  sumc(ti);
"Number of Individuals:  "  rows(ti);
"Regressors X:           "; $x_names';
"Right-hand-side Vars:   "; $w_names';
"";
"";


/********************************************************
**      Configure the prior for the nuisance parameters
**
**      specify baseline ind: age_base, educ_base, gender_base, hours_base     
**                            xbaseline
**      individual effect:    beta_m0, beta_v0
**      covariates       :    gamss_m0, gamss_v0
*/
#include c:\projects\active\persist\gauss\prog_p03\confprio.g;
gamss_v0 = gamss_v0*scale_g^2;
beta_v0  = beta_v0*scale_b^2;
"Prior successfully configured";
"";

output file = ^logfile1 on;
"Baseline Individual";
"===================";
"AGE:                         " AGE_B;
"EDUCATION:                   " EDUC_B;
"GENDER (0 is MALE):          " GENDER_B;
"HOURS:                       " HOURS_B;
"EXP(Prior Mean lWAGE (1976)):" exp(lwage76_m);
"Prior Mean WAGE (1976):      " wage76_m;
"Prior Mean Ind Effect:       " specific_m;
"";
"Scaling factors for covariance matrices:  " scale_b~scale_g;
"S.D. for individual specific effect is:   " sqrt(beta_v0);
"";
"";
output file = ^logfile1 off;

/******************************************************** 
**      Estimate the Panel Wage equation
**
*/ 

npara  = 4;

/* Starting Values for Maximization
*/
rho =    0.8;
mu  =    0.5;
phi =    0.5; 
sigeps = 0.2;

para       = rho | mu | phi | sigeps;
para_names = "RHO" | "MU" | "PHI" | "SIG";

/* Set path and import the information for the prior density
*/

lpath = "c:\\projects\\active\\persist\\gauss\\prog_p03\\para";

priorfile = "panelpri.out";
load path=^lpath prior[npara,3] = ^priorfile;

pmean  = prior[.,1];
pstdd  = prior[.,2];
pshape = prior[.,3];

/******************************************************** 
**     Load transformation scheme for parameters
**
*/
transfile = "trans.out";
load path=^lpath _trspec[npara,4] = ^transfile;


/******************************************************** 
**      Calculate posterior density at starting values
**
*/
{gam1_m0, gam11_v0, gam0_m0, gam00_v0, gam01_v0} = qdiff_gam(para, gamss_m0, gamss_v0, mspec); 
{lnpy, gam1_mT, gam11_vT}   = loglhpanel( para, beta_m0, beta_v0, gam1_m0, gam11_v0);

"Posterior at Starting Value:    " lnpy;
"Prior Density at Starting Value:" priodens(para,pmean,pstdd,pshape);
"";
"Press Key to Continue";
w = keyw;
cls;

/******************************************************** 
**   Check Consistency of the Parameter Transformation
**
*/
{gamss_mT, gamss_vT, AEP_mT, AEP_vT} 
  = qdiffinv_gam(para, gam1_m0, gam11_v0, gam0_m0, gam00_v0, gam1_m0, gam11_v0, gam01_v0, mspec);

format /m0 /rds 10,4;
"Transformation Check";
"====================";
"Parameter  :  Original | Transformed";   

if mspec == 2;
   "";
   "AEP - Scale:"  AEPscale_m "|" AEP_mT[1];
   "AEP - Peak :"  AEPpeak_m  "|" AEP_mT[2];
   "Return to S:"  retedu_m   "|" gamss_mT[3];
   "Gender Bias:"  gender_m   "|" gamss_mT[4];
endif;

wage_baseline0 = lwage76_m+seqa(0,wagegr_m,nydummies+1);
" ";
"Wage 1976:  " exp(wage_baseline0[1])   "|" exp(gamss_mT[rows(gamss_mT)-nydummies]);
"Wage 1980:  " exp(wage_baseline0[1+4]) "|" exp(gamss_mT[rows(gamss_mT)-nydummies+4]);
"Wage 1985:  " exp(wage_baseline0[1+9]) "|" exp(gamss_mT[rows(gamss_mT)-nydummies+9]);
"Wage 1990:  " exp(wage_baseline0[1+14]) "|" exp(gamss_mT[rows(gamss_mT)-nydummies+14]);

"";
"Press Key to Continue";
w = keyw;
cls;


/********************************************************
**       Maximize the posterior density
**
*/
format /m1 /ro 16,8;
x0= invtrans(para)+0.3*rndn(npara,1);
H0= eye(npara)*1E-4;
nit = 1000;
crit= 1E-8;

/* Use personal optimization procedure
*/
{fh,xh,g,H,itct,fcount,retcode} = csminwel(x0,H0,crit,nit,&objfcn);

/* Save parameter estimates and print to screen
*/
paraest = trans(xh);
lbdmod_p = paraest;

save path = ^lpath lbdmod_p, para_names;

/********************************************************
**       Estimation Output
**
*/
cls;
{gam1_m0, gam11_v0, gam0_m0, gam00_v0, gam01_v0} = qdiff_gam(paraest, gamss_m0, gamss_v0, mspec); 
{lnpy, gam1_mT, gam11_vT}   = loglhpanel( paraest, beta_m0, beta_v0, gam1_m0, gam11_v0);
lnprio = priodens( paraest, pmean, pstdd, pshape);

/* Compute gradient in terms of model parameters
*/
gradmod = g./ diag(gradp(&invtrans,paraest));

output file = ^logfile1 on;
"Posterior Mode     " (lnpy+lnprio);
"Likelihood at Mode " lnpy;
"Prior at Mode      " lnprio;
" ";
"Parameter   | Estimate |  Prior  | Start-V  | Gradient";
outmat = para_names~paraest~pmean~trans(x0)~gradmod;
let mask[1,5] = 0 1 1 1 1;
let fmt[5,3] =
   "-*.*s " 10 4
   "*.*lf " 10 4
   "*.*lf " 10 4
   "*.*lf " 10 4
   "*.*lf " 10 4;

d = printfm(outmat,(0 ~ 1 ~ 1 ~ 1 ~ 1),fmt);

{gamss_mT, gamss_vT, AEP_mT, AEP_vT} 
  = qdiffinv_gam(paraest, gam1_mT, gam11_vT, gam0_m0, gam00_v0, gam1_m0, gam11_v0, gam01_v0, mspec);

format /m0 /rds 10,4;
mu_q = paraest[2]*(1-paraest[3]^(1/4))/(1-paraest[3]);
phi_q = paraest[3]^(1/4);

"";
"Time Series Parameters";
"======================";
"";
"MU(Quarterly):  " mu_q;
"PHI(Quarterly): " phi_q;
"";

"Nuisance Parameters ";
"====================";
"Parameter  :  Prior | Estimated";   

if mspec == 2;
   "";
   "AEP - Scale:"  AEPscale_m "|" AEP_mT[1];
   "AEP - Peak :"  AEPpeak_m  "|" AEP_mT[2];
   "Return to S:"  retedu_m   "|" gamss_mT[3];
   "Gender Bias:"  gender_m   "|" gamss_mT[4];
   "";
endif;

wage_baseline0 = lwage76_m+seqa(0,wagegr_m,nydummies+1);
" ";
"Wage 1976:  " exp(wage_baseline0[1])   "|" exp(gamss_mT[rows(gamss_mT)-nydummies]);
"Wage 1980:  " exp(wage_baseline0[1+4]) "|" exp(gamss_mT[rows(gamss_mT)-nydummies+4]);
"Wage 1985:  " exp(wage_baseline0[1+9]) "|" exp(gamss_mT[rows(gamss_mT)-nydummies+9]);
"Wage 1990:  " exp(wage_baseline0[1+14]) "|" exp(gamss_mT[rows(gamss_mT)-nydummies+14]);
"";
"****************************************************************";
"";
"";
output file = ^logfile1 off;

output file = ^logfile2 on;
screen off;
(mspec ~ scale_b ~ scale_g ~ wage76_m ~ paraest[1] ~ mu_q ~ phi_q);
output file = ^logfile2 off;
screen on;
end;

/****************************************************/
/*                 PROCEDURES                       */
/****************************************************/

proc (1) = objfcn(para);
/* Evaluate the product of likelihood and prior
*/
local modpara, gam1_m, gam11_v, gam0_m, gam00_v, gam01_v, lnpy, gam1_mT, gam11_vT, lnprio;

  modpara = trans(para); 
  {gam1_m, gam11_v, gam0_m, gam00_v, gam01_v} = qdiff_gam(modpara, gamss_m0, gamss_v0, mspec); 
  {lnpy, gam1_mT, gam11_vT} = loglhpanel( modpara, beta_m0, beta_v0, gam1_m, gam11_v);
  
/* Evaluate the Prior density
*/      
  lnprio = priodens( modpara, pmean, pstdd, pshape);
  locate 24,35; "prior     " lnprio; 
  locate 25,35; "Likelihood" lnpy;
    
retp(real(-lnpy-lnprio));  /* We minize the inverse of the likelihood fcn */
endp;


