/****************************************************/
/*                                                  */
/*       Likelihood*Prior for the DSGE Model        */
/*                                                  */
/****************************************************/

/* filename:    lbdpm.g
** description: The program computes the likelihood * prior for the
**              linearized DSGE model
** created:     05/30/01
*/

new;
closeall;
library pgraph, lbdlib, user;
cls; 
_FIX1 = 0;
_FIX2 = 0;
__output = 1;
outwidth 128;

/******************************************************** 
**      Estimate the DSGE Model
**      Models: 1 = RBC
**              2 = LBD
**              3 = LBD + Effort
*/ 
mspec  = 1;
mprior = 1;
npara  = 12;

/******************************************************** 
** Import data on output growth and inflation: series (nobs,2)
** observations from 1954:III to 1997:IV: 
**
** YY is composed of gdpq_cld and blsh_cl
*/
#include c:\projects\active\persist\Gauss\prog_t03\loaddata.g


/******************************************************** 
** Starting Values for Maximization
*/
alp = 0.6;
bet = 0.99;
gam = 0.004;
del = 0.025;
nu  = 2;
rho = 0.98;
mu  = 0.1;
phi = 0.8;
chi = 0.5;
psi1 = 3;
siga = 0.1;
sigb = 0.1;

para = alp | bet | gam | del | nu | rho | 
       mu | phi |
       chi | 
       psi1 | siga | sigb ;

para_names = { ALP BET GAM DEL NU RHO MU PHI CHI PSI SIGA SIGB };
save path = ^lpath para_names;

/* Load transformation scheme for parameters
*/
transfile = "trans.out";
load path=^lpath _trspec[npara,4] = ^transfile;

/******************************************************** 
**      Calculate posterior density at starting values
**
*/
lnpy = -objfcn(invtrans(para));
cls;
"Posterior at Starting Value:    " lnpy;
"Prior Density at Starting Value:" priodens(para,pmean,pstdd,pshape) priomuphi(para);
"Press key to continue";
w = keyw;
cls;

/********************************************************
**       Maximize the posterior density
**
*/
x0= invtrans(para)+0.1*rndn(npara,1);
H0= eye(npara)*1E-4;
nit = 1000;
crit= 1E-9;

/* Use personal optimization procedure
*/
{fh,xh,g,H,itct,fcount,retcode} = csminwel(x0,H0,crit,nit,&objfcn);

/********************************************************
** Save parameter estimates
*/
paraest = trans(xh);
if mspec == 1;
   paraest[7:9] = zeros(3,1);
elseif mspec == 2;
   paraest[9] = 0;
endif;

oparaest         = lpath $+ "\\" $+ lmodel $+ lprior $+ "mode";
create fhparaest =^oparaest with PARAEST, 1, 8;
wr               = writer(fhparaest,paraest);
closeall fhparaest;   


/********************************************************
**       Estimation Output
**
*/
{lnpy, obsmean, obsvar} = evallbd( paraest,mspec,T0,YY);
lnprio1 = priodens( paraest, pmean, pstdd, pshape);
lnprio2 = priomuphi( paraest );

/* Compute gradient in terms of model parameters
*/
gradmod = g./ diag(gradp(&invtrans,paraest));

cls;
"Posterior Mode     " (lnpy+lnprio1+lnprio2);
"Likelihood at Mode " lnpy;
"Prior1 at Mode     " lnprio1;
"Prior2 at Mode     " lnprio2;
"Mu-Phi Scaling     " muphi_scale;
" ";
"Parameter   | Estimate |  Start-V  | Prior Mean | Prior Stdd | Gradient";
outmat = para_names'~paraest~trans(x0)~pmean~pstdd~gradmod;
let mask[1,6] = 0 1 1 1 1 1;
let fmt[6,3] =
   "-*.*s " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4;

d = printfm(outmat,(0~1~1~1~1~1),fmt);
end;


/****************************************************/
/*                 PROCEDURES                       */
/****************************************************/

proc (1) = objfcn(para);
local lnpY, lnpY0, obsmean, obsvar, lnprio1, lnprio2, modpara;

/* likelihood of the full data vector
*/
 
 modpara = trans(para); 

 {lnpy, obsmean, obsvar} = evallbd( modpara,mspec,T0,YY);
  
/* Evaluate the Prior density
*/      
  lnprio1 = priodens( modpara, pmean, pstdd, pshape);
  lnprio2 = priomuphi( modpara );
    
retp(real(-lnpy-lnprio1-lnprio2));  /* We minize the inverse of the likelihood fcn */
endp;

/***************************************************************************
*/

proc (1) = priomuphi(para);
local muphi, lnprio;
muphi = para[7:8];
if mspec > 1;
   lnprio = -ln(2*pi) - 0.5*ln(det(muphi_v0))
            - 0.5*(muphi - muphi_m0)'*inv(muphi_v0)*(muphi - muphi_m0);
else;
   lnprio = 0;
endif;
retp(lnprio);
endp;
